#ifndef MTRAND_H_INCLUDED
#define MTRAND_H_INCLUDED

#include <cassert>
#include <ctime>

//////////////////////////////////////////////////////////////////////////

// fr Visual Studio
#if defined(_MSC_VER)
typedef unsigned __int8 uint8_t;
typedef signed __int8 int8_t;
typedef unsigned __int16 uint16_t;
typedef signed __int16 int16_t;
typedef unsigned __int32 uint32_t;
typedef signed __int32 int32_t;
#endif

//////////////////////////////////////////////////////////////////////////

// Mersenne Twister-Zufallsgenerator
class MTRand
{
public:
    MTRand(uint32_t seed = 0) { this->seed(seed); }

    void seed(uint32_t seed)
    {
        next = 0;
        mt[0] = seed;
        for(uint32_t i = 1; i != 624; ++i) mt[i] = static_cast<uint32_t>(69069ull * mt[i - 1] + 1);
    }

    uint32_t operator ()()
    {
        if(next == 624) generateNumbers(), next = 0;
        uint32_t x = mt[next++];
        x ^= x >> 11;
        x ^= (x << 7) & 0x9D2C5680;
        x ^= (x << 15) & 0xEFC60000;
        x ^= x >> 18;
        return x;
    }

private:
    void generateNumbers()
    {
        const uint32_t xorTab[] = { 0, 0x9908B0DF };
        for(uint32_t i = 0; i != 227; ++i) { const uint32_t x = (mt[i] & 0x80000000) | (mt[i + 1] & 0x7FFFFFFF); mt[i] = mt[i + 397] ^ (x >> 1) ^ xorTab[x & 1]; }
        for(uint32_t i = 227; i != 624; ++i) { const uint32_t x = (mt[i] & 0x80000000) | (mt[i + 1] & 0x7FFFFFFF); mt[i] = mt[i - 227] ^ (x >> 1) ^ xorTab[x & 1]; }
        const uint32_t x = (mt[623] & 0x80000000) | (mt[0] & 0x7FFFFFFF);
        mt[623] = mt[396] ^ (x >> 1) ^ xorTab[x & 1];
    }

    uint32_t mt[624];
    uint32_t next;
};

//////////////////////////////////////////////////////////////////////////

// reserviert Speicher und richtet ihn an einer 16-Byte grenze aus
template<typename T> inline T* aligned_alloc(uint32_t num_elements)
{
    assert(num_elements);
    const uint32_t num_dwords = (num_elements * sizeof(T) + 3) >> 2;
    uint32_t* const p_raw_memory_32 = new uint32_t[num_dwords + 4];
    const uint32_t offset = 4 - ((p_raw_memory_32 - reinterpret_cast<uint32_t*>(0)) & 3);
    p_raw_memory_32[offset - 1] = p_raw_memory_32 - reinterpret_cast<uint32_t*>(0);
    return reinterpret_cast<T*>(p_raw_memory_32 + offset);
}

//////////////////////////////////////////////////////////////////////////

// gibt ausgerichteten Speicher frei
template<typename T> inline void aligned_free(T* p_memory)
{
    assert(!((reinterpret_cast<uint8_t*>(p_memory) - reinterpret_cast<uint8_t*>(0)) & 15));
    uint32_t* const p_memory_32 = reinterpret_cast<uint32_t*>(p_memory);
    uint32_t* const p_raw_memory_32 = reinterpret_cast<uint32_t*>(0) + p_memory_32[-1];
    assert(p_memory_32 > p_raw_memory_32 && p_memory_32 <= p_raw_memory_32 + 4);
    delete[] p_raw_memory_32;
}

//////////////////////////////////////////////////////////////////////////

// Zeitmessung
inline double time()
{
	return static_cast<double>(clock()) / CLOCKS_PER_SEC;
}

#endif
